////////////////////////////////////////////////////////////
//Binärbaum mit LSystem erzeugt. Kapitel Turtlegrafik
/////////////////////////////////////////////////////////////

int generations = 1;      // Anzahl Ersetzungen (Rekursionstiefe)
float len = 40;           // Länge eines Segments in Pixeln, Anfangswinkel
float delta = -15;        // Veraenderung im Winkel

String axiom = "F";
String substitut = "F[+F][-F]";
String s = axiom;


void maleStrich() {
  line(0, 0, len, 0);
  translate(len, 0);
}

void drehePositiv() {
  rotate(radians(delta));
}

void dreheNegativ() {
  rotate(radians(-delta));
}

void speichereZustand() {
  pushMatrix();
}

void restauriereZustand() {
  popMatrix();
}

void applySubstitutions() {
  
  s = axiom;
  // ersetze alle "F" in s durch das Substitut
  for (int i=0; i<generations; i++) 
     s = s.replaceAll("F",substitut);
     println(s); 
     println("--------------");
}

void setup() 
{
  size(800, 800);
  smooth();
  noLoop();
  applySubstitutions();
  redraw();
}

void draw() {
  background(0);
  stroke(255);
  translate(width/2,9*height/10);
  rotate(radians(-90));
  println(generations);//Zur Kontrolle
  for (int i=0;i<s.length();i++) {
    char c = s.charAt(i);
    switch (c) {
      case 'F': maleStrich();
                break;
      case '+': drehePositiv();
                break;
      case '-': dreheNegativ();
                break;
      case '[': speichereZustand();
                break;
      case ']': restauriereZustand();
                break;
    }  
  }
}

void keyPressed() {
  switch (key) {
    case '+' :
      generations = min(14,generations+1);
      applySubstitutions();
      redraw();
      break;
    case '-':
      generations = max(1,generations-1);
      applySubstitutions();
      redraw();
      break;
    case 'r':
      len *= 0.66;
      redraw();
      break;
    case 'e':
      len /= 0.66;
      redraw();
      break;
  }     
}
